import pyspark.sql.functions as F
from lhcsmapi.Time import Time
from datetime import datetime


def path_exists(path, sc):
    fs = sc._jvm.org.apache.hadoop.fs.FileSystem.get(
        sc._jsc.hadoopConfiguration(),
    )
    return fs.exists(sc._jvm.org.apache.hadoop.fs.Path(path))


def get_checkpoint_timestamp(path, spark, sc):
    if not path_exists(path, sc):
        return False

    df = spark.read.csv(path=path, header=True)
    return int(df.select(F.max(F.col('checkpoint_timestamp')).alias('LATEST')).limit(1).collect()[0].LATEST)


def save_row_with_checkpoint(pandas_row, path, t_checkpoint, spark, sc):
    if path_exists(path, sc):
        df = spark.read.csv(path=path, header=True)
        checkpoint_exists = (
                    df.select('checkpoint_timestamp').where(F.col('checkpoint_timestamp') == t_checkpoint).count() != 0)
        if checkpoint_exists:
            print("WARNING: Checkpoint at %d already exists, skipping" % (t_checkpoint))
            return False

    df = spark.createDataFrame(pandas_row)
    df = df.withColumn('checkpoint_creation', F.lit(Time.to_unix_timestamp(datetime.now())))
    df = df.withColumn('checkpoint_timestamp', F.lit(t_checkpoint))

    df.write.csv(path=path, mode="append", header=True)

    return True