from lhcsmapi.metadata.SignalMetadata import SignalMetadata
from lhcsmapi.metadata.MappingMetadata import MappingMetadata
from lhcsmapi.Time import Time
from lhcsmapi.dbsignal.Signal import Signal
from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
from lhcsmapi.dbsignal.SignalUtilities import SignalUtilities


class DiodeLeadResistanceQuery(object):

    @staticmethod
    def get_magnet_type_i_meas_u_diode(magnet, circuit_name):
        circuit_type = circuit_name[0:2]

        magnet_type = "M" + circuit_type[1:2]

        i_meas = SignalMetadata.get_signal_name(circuit_type, circuit_name, system="PC", database="CALS",
                                                signal="I_MEAS")
        u_diode = SignalMetadata.get_signal_name(circuit_type, circuit_name, system="DIODE_{}".format(circuit_name.split('.')[0]),
                                                 database='CALS', signal="U_DIODE_{}".format(circuit_name.split('.')[0]))
        u_diode = u_diode.replace("%MAGNET%", "{}.{}".format(magnet_type, magnet))

        return magnet_type, i_meas, u_diode

    @staticmethod
    def query_cals_signal(signal_name, timestamp, magnet_type, *, ldb=None):
        # get start and end time
        timestamp_unix_sec = Time.to_unix_timestamp_in_sec(timestamp)
        t_end = 350 if magnet_type == "MB" else 150

        # query signal
        signal = Signal().read('CALS', signal=signal_name, t_start=(timestamp_unix_sec, 's'),
                               duration=[(50, 's'), (t_end, 's')], ldb=ldb)
        return SignalUtilities.synchronize_df(signal, timestamp_unix_sec)


    @staticmethod
    def find_diode_pm_entries(circuit_type, circuit_name, system_diode, magnet, timestamp_iqps_qh, duration=[(600, 's'), (600, 's')]):
        metadata_diode = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_name, system_diode,
                                                                           'PM')
        class_name_qps, system_qps = metadata_diode['className'], metadata_diode['system']
        source_qps = MappingMetadata.get_crate_name_from_magnet_name(circuit_type, magnet)

        event_source_timestamp_nqps = PmDbRequest.find_events(source_qps, system_qps, class_name_qps,
                                                              t_start=timestamp_iqps_qh, duration=duration)

        return [timestamp for _, timestamp in event_source_timestamp_nqps]

    @staticmethod
    def find_pc_pm_entries(circuit_type, circuit_name, timestamp_iqps_qh, duration=[(600, 's'), (600, 's')]):
        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_name, 'PC', 'PM')
        class_name_fgc, source_fgc, system_fgc = metadata_fgc['className'], metadata_fgc['source'], metadata_fgc[
            'system']

        event_source_timestamp_pc = PmDbRequest.find_events(source_fgc, system_fgc, class_name_fgc,
                                                            t_start=timestamp_iqps_qh, duration=duration)

        return [timestamp for _, timestamp in event_source_timestamp_pc]

    def query_iab_i_a(circuit_type, circuit_name, timestamp_fgc):
        metadata_fgc = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_name, 'PC', 'PM')
        class_name_fgc, source_fgc = metadata_fgc['className'], metadata_fgc['source']
        I_A = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'PC', 'PM', 'I_A')

        return Signal().read('pm', system='FGC', className=class_name_fgc, source=source_fgc,
                             signal=I_A, eventTime=timestamp_fgc)

    @staticmethod
    def query_u_diode(circuit_type, circuit_name, magnet, timestamp_nqps):
        # get metadata
        magnet_type, i_meas_name, u_diode_name = DiodeLeadResistanceQuery.get_magnet_type_i_meas_u_diode(magnet,
                                                                                                         circuit_name)
        # get source qps
        source_qps = MappingMetadata.get_crate_name_from_magnet_name(circuit_type, magnet)

        # query U_DIODE
        system_diode = 'DIODE_RB' if circuit_type == "RB" else 'DIODE_RQD'
        metadata_diode = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_name, system_diode,
                                                                           'PM')

        return Signal().read('pm', system=metadata_diode['system'], className=metadata_diode['className'],
                             source=source_qps, signal=u_diode_name, eventTime=timestamp_nqps)

    @staticmethod
    def query_u_ref(circuit_type, circuit_name, magnet, timestamp_nqps):
        # get source qps
        source_qps = MappingMetadata.get_crate_name_from_magnet_name(circuit_type, magnet)

        # query U_DIODE
        system_diode = 'DIODE_RB' if circuit_type == "RB" else 'DIODE_RQD'

        u_ref_name = SignalMetadata.get_signal_name(circuit_type, circuit_name, system_diode, 'PM', 'U_REF_N1').replace(
            '%QPS_CRATE%', source_qps)
        metadata_diode = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_name, system_diode,
                                                                           'PM')
        return Signal().read('pm', system=metadata_diode['system'], className=metadata_diode['className'],
                             source=source_qps, signal=u_ref_name, eventTime=timestamp_nqps)