import matplotlib.pyplot as plt

from lhcsmapi.Time import Time


class QuenchHeaterPlot(object):

    @staticmethod
    def plot_qh_discharge(circuit_type, qh_sync_decay_df, qh_ref_sync_decay_df, timestamp_qh, source_qh):
        no_subplots = 3 if circuit_type == "RB" else 1

        fig, ax = plt.subplots(1, no_subplots, figsize=(20, 7))
        qh_sync_decay_df.filter(regex=("U_HDS_\d{1}")).plot(ax=ax[0])
        qh_ref_sync_decay_df.filter(regex=("U_HDS_\d{1}")).plot(ax=ax[0], style='--')
        ax[0].grid(True)
        ax[0].tick_params(labelsize=15)
        ax[0].set_xlabel("time, [s]", fontsize=15)
        ax[0].set_ylabel("U_HDS, [V]", fontsize=15)

        if circuit_type == "RB":
            title = "Magnet: {}, Time Stamp: {}, U_HDS(t), I_HDS(t), and R_HDS(t)".\
                format(source_qh, Time.to_string_short(timestamp_qh))

            qh_sync_decay_df.filter(regex=("I_HDS_\d{1}")).plot(ax=ax[1], title=title)
            ax[1].title.set_size(20)
            qh_ref_sync_decay_df.filter(regex=("I_HDS_\d{1}")).plot(ax=ax[1], style='--')
            ax[1].grid(True)
            ax[1].tick_params(labelsize=15)
            ax[1].set_xlabel("time, [s]", fontsize=15)
            ax[1].set_ylabel("I_HDS, [A]", fontsize=15)

            qh_sync_decay_df.filter(regex=("R_HDS_\d{1}")).plot(ax=ax[2])
            qh_ref_sync_decay_df.filter(regex=("R_HDS_\d{1}")).plot(ax=ax[2], style='--')
            ax[2].grid(True)
            ax[2].tick_params(labelsize=15)
            ax[2].set_xlabel("time, [s]", fontsize=15)
            ax[2].set_ylabel("R_HDS (Calculated), [Ohm]", fontsize=15)

        plt.show()
