from lhcsmapi.dbsignal.Signal import Signal
from lhcsmapi.dbsignal.SignalElectrical import SignalElectrical
from lhcsmapi.dbsignal.SignalUtilities import SignalUtilities
from lhcsmapi.dbsignal.ExpSignalAnalysis import ExpSignalAnalysis
from lhcsmapi.metadata.SignalMetadata import SignalMetadata
from lhcsmapi.reference.Reference import Reference


class QuenchHeaterQuery(object):

    @staticmethod
    def get_actual_reference_discharges(circuit_type, circuit_name, source_qds, timestamp_qds):
        qh_df = QuenchHeaterQuery.query_qh_discharge(circuit_type, circuit_name, source_qds, timestamp_qds)
        qh_sync_decay_df = QuenchHeaterQuery.preprocess_qh_discharge(circuit_type, circuit_name, qh_df,
                                                                     timestamp_qds, source_qds)

        qh_ref_discharges = Reference.read_quench_heater_reference_discharges(circuit_type)
        timestamp_qh_ref = qh_ref_discharges[qh_ref_discharges["Cell"] == source_qds]["Timestamp"].values[0]
        qh_ref_df = QuenchHeaterQuery.query_qh_discharge(circuit_type, circuit_name, source_qds, timestamp_qh_ref)
        qh_ref_sync_decay_df = QuenchHeaterQuery.preprocess_qh_discharge(circuit_type, circuit_name, qh_ref_df,
                                                                         timestamp_qh_ref, source_qds)

        return qh_sync_decay_df, qh_ref_sync_decay_df

    @staticmethod
    def query_qh_discharge(circuit_type, circuit_name, source_qh, timestamp_qh):
        # Metadata
        U_HDS = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'QH', 'PM', 'U_HDS')
        if circuit_type == 'RB':
            I_HDS = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'QH', 'PM', 'I_HDS')
            I_HDS = [i.replace('%CELL%', source_qh) if '%CELL%' in i else i for i in I_HDS]
        else:
            I_HDS = []

        U_HDS = [u.replace('%CELL%', source_qh) if '%CELL%' in u else u for u in U_HDS]


        metadata_qh = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_name, 'QH', 'PM')
        system_qh, classname_qh = metadata_qh['system'], metadata_qh['className']

        # query post-mortem
        qh_signal = Signal()
        for qh_signal_name in U_HDS + I_HDS:
            qh_signal.read('pm', append=True, system=system_qh, className=classname_qh, source=source_qh,
                           signal=qh_signal_name, eventTime=timestamp_qh)

        return qh_signal.df

    @staticmethod
    def preprocess_qh_discharge(circuit_type, circuit_name, qh_df, timestamp_qh, magnet):

        # Synchronize time to 0
        qh_sync_df = SignalUtilities.synchronize_df(qh_df, timestamp_qh)
        # Convert indices to seconds
        qh_sync_df = SignalUtilities.convert_indices_to_sec(qh_sync_df)
        # Extract decay only
        qh_sync_decay_df = ExpSignalAnalysis.get_df_with_decay_only(qh_sync_df)
        # Filter
        qh_sync_decay_df = qh_sync_decay_df.rolling(window=3, min_periods=1).median()
        # Calculate resistance
        if circuit_type == "RB":
            # Get metadata
            U_HDS = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'QH', 'PM', 'U_HDS')
            U_HDS = [U_HDS_el.replace('%CELL%', magnet) for U_HDS_el in U_HDS]
            I_HDS = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'QH', 'PM', 'I_HDS')
            I_HDS = [I_HDS_el.replace('%CELL%', magnet) for I_HDS_el in I_HDS]
            R_HDS = [u.replace('U', 'R') for u in U_HDS]
            qh_sync_decay_df = SignalElectrical.calculate_resistances(qh_sync_decay_df, U_HDS, I_HDS, R_HDS)

        return qh_sync_decay_df

