from abc import ABC, abstractmethod

class DbSignal(ABC):
    """ Base class for encapsulating the read and write methods of signal databases
    """

    def __init__(self, name=''):
        self.signal = name

    def write(self, signal=None, **kwargs):
        """ Base implementation for write method. Defaults to telling the user, the api can't be written to.
        """

        print("Can not write to " + self.signal)
    
    @abstractmethod
    def read(self, signal=None, **kwargs):
        pass
    
    @staticmethod
    @abstractmethod
    def build_signal(**kwargs):
        pass

    @staticmethod
    def validate_key_in_kwargs(key, *key_types, **kwargs):
        result = None
        if key in kwargs:
            result = kwargs[key]
            match_found = False
            for key_type in key_types:
                match_found = match_found or isinstance(result, key_type)
            if not match_found:
                raise TypeError("You entered a {} keyword, but the keyword needs to be a {}.".format(key, key_types))

        return result