import pandas as pd

from lhcsmapi.Time import Time

# ToDo - move to pyeDSL
class SignalUtilities(object):
    """SignalUtilities class contains static methods for processing dataframe indices
    """
    
    @staticmethod
    def convert_indices(data_structure, conversion_function, unit='ns', tz=None):
        """Method applies a conversion function to the index of the signal's DataFrame
        """

        if isinstance(data_structure, (pd.DataFrame, pd.Series)):
            if not data_structure.empty:
                index = data_structure.index.tolist()
                new_index = [conversion_function(t, unit, tz) for t in index]
                data_structure.set_index([new_index], inplace=True)
            return data_structure
        else:
            raise TypeError("""Cannot convert the indices of input {}.
                            The data structure must be a pandas DataFrame or Series.""".format(type(data_structure)))

    @staticmethod
    def convert_indices_to_unix_timestamp(data_structure, unit='ns', tz=None):
        """Method coverts the indices of a pandas DataFrame to unix time (ns) and returns the DataFrame
        """
        return SignalUtilities.convert_indices(data_structure, Time.to_unix_timestamp, unit, tz)
    
    @staticmethod
    def convert_indices_to_sec(data_structure, unit='ns', tz=None):
        """Method coverts the indices of a pandas DataFrame to unix time (s) and returns the DataFrame
        """
        return SignalUtilities.convert_indices(data_structure, Time.to_unix_timestamp_in_sec, unit, tz)

    @staticmethod
    def convert_indices_to_timestamps(data_structure, unit='ns', tz=None):
        """Method coverts the indices of a pandas DataFrame to pandas Timestamps and returns the DataFrame
        """
        return SignalUtilities.convert_indices(data_structure, Time.to_pandas_timestamp, unit, tz)

    @staticmethod
    def synchronize_df(df, t0=None):
        """Method coverts the indices of a pandas DataFrame to unix time (s) and synchronizes them then returns the DataFrame
        """
        t0 = df.index[0] if t0 is None else t0

        new_index = [idx - t0 for idx in df.index]
        df.set_index([new_index], inplace=True)
        return df

