import re
import pandas as pd

from lhcsmapi.Time import Time


class AftDbRequest(object):
    _url_prefix = 'https://aft.cern.ch/api/public/v1/'
    _id_reg_exp = '(?<={)(.*?)(?=})'

    def __init__(self, session):
        self.session = session

    def context_query(self, context, context_id=None, **kwargs):
        if any([context == c for c in ['accelerators', 'faults/labels', 'faults/states', 'faulty-element-types']]):
            url = self._url_prefix + context

            response = self.session.get(url, verify=False)
            return pd.DataFrame(response.json())
        elif any([context == c for c in
                  ['accelerators/{acceleratorId}/properties', 'accelerators/{acceleratorId}/systems',
                   'systems/{systemId}/properties']]):
            url = self._url_prefix + context
            found_id = re.findall(self._id_reg_exp, url)[0]
            url = url.replace('{%s}' % found_id, str(context_id))
            response = self.session.get(url, verify=False)

            return pd.DataFrame(response.json())
        elif context == 'faulty-elements/{faultyElementId}/statistics':
            url = self._url_prefix + context
            found_id = re.findall(self._id_reg_exp, url)[0]
            url = url.replace('{%s}' % found_id, str(context_id))
            response = self.session.get(url, verify=False)

            return pd.DataFrame(response.json(), index=[context_id])
        elif context == 'faults/{faultId}':
            url = self._url_prefix + context
            found_id = re.findall(self._id_reg_exp, url)[0]
            url = url.replace('{%s}' % found_id, str(context_id))
            response = self.session.get(url, params=kwargs, verify=False)

            return response.json()
        elif context == 'faulty-elements':
            url = '%s%s?' % (self._url_prefix, context)
            response = self.session.get(url, params=kwargs, verify=False)

            return pd.DataFrame(response.json())
        else:
            raise ValueError(
                'Context %s is not supported by AFT REST API. '
                'For more details, please consult https://aft.cern.ch/docs' % context)

    def fault_query(self, t_start, t_end, **kwargs):
        time_def = {'startTime': Time.to_string_short(t_start).replace(' ', 'T')+'Z',
                    'endTime': Time.to_string_short(t_end).replace(' ', 'T')+'Z'}
        params = {**time_def, **kwargs}

        url = self._url_prefix + 'faults?'
        response = self.session.get(url, params=params, verify=False)

        return pd.DataFrame(response.json())

