from lhcsmapi.Time import Time
from lhcsmapi.metadata.SignalMetadata import SignalMetadata
from lhcsmapi.dbsignal.post_mortem.PmDbRequest import PmDbRequest
from lhcsmapi.gui.pc.RbFgcPmSearchBaseModule import FgcPmSearchBaseModule


def check_quench_presence_around_fqc_timestamp(circuit_type, circuit_name, source_timestamp_qds):
    source_timestamp_qds_filtered_act = []
    for source, timestamp in source_timestamp_qds:
        rq_circuits_with_cell = SignalMetadata.get_sector_names_from_components(circuit_type, 'MAGNET', source)
        # ['RQD.A12', 'RQF.A12'] -> 'RQD.A12' -> ['RQD', 'A12'] -> 'A12'
        rq_sector_with_cell = rq_circuits_with_cell[0].split('.')[1]
        rq_sector_with_circuit_name = circuit_name.split('.')[1]
        if rq_sector_with_cell == rq_sector_with_circuit_name:
            source_timestamp_qds_filtered_act.append((source, timestamp))
    return source_timestamp_qds_filtered_act


class RqFgcPmSearchBaseModule(FgcPmSearchBaseModule):

    def get_circuit_names(self, index, fgc_pm_entries):
        return sorted(list([fgc_pm_entries[index][0], fgc_pm_entries[index][2]]))

    def convert_fgc_pm_entries_to_options(self, fgc_pm_entries):
        return ['{}: {}; {}: {}'.format(x[0], Time.to_string(x[1]), x[2], Time.to_string(x[3])) for x in fgc_pm_entries]

    def find_fgc_pm_entries_with_qps(self, start_time, end_time, circuit_name):
        # convert RQ.AXX into [RQD.AXX, RQF.AXX]
        circuit_names = [circuit_name.replace('RQ', rq_circuit_name) for rq_circuit_name in ['RQD', 'RQF']]

        source_timestamp_fgc = self._find_fgc_pm_entries(start_time, end_time, 'RQ', circuit_names)
        return self._keep_fgc_rqd_with_qps_and_rqf('RQ', circuit_names, source_timestamp_fgc)

    def _find_fgc_pm_entries(self, start_time, end_time, circuit_type, circuit_names):

        metadata_fgc_rqd = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_names[0], 'PC',
                                                                               'PM')

        quotient, remainder = Time.modulo_day_in_unix_timestamp(start_time, end_time)
        event_duration_nsec = int(24 * 60 * 60 * 1e9)
        source_timestamp_fgc = []

        self.progress_search.layout.visibility = 'visible'
        self.progress_search.max = quotient
        self.progress_search.description = f'Days: 0/{quotient}'

        for day in range(quotient):
            start_time_temp = Time.to_unix_timestamp(start_time) + int(day * event_duration_nsec)

            # Update GUI
            self.lbl_search.value = 'Querying PM for FGC entries on {}'.format(Time.to_string(start_time_temp))

            source_timestamp_fgc += PmDbRequest.find_events(metadata_fgc_rqd['source'], metadata_fgc_rqd['system'],
                                                            metadata_fgc_rqd['className'], t_start=start_time_temp,
                                                            duration=[(event_duration_nsec, 'ns')])
            self.progress_search.description = f'Days: {day + 1}/{quotient}'
            self.progress_search.value = day + 1

        if remainder > 0:
            start_time_temp = Time.to_unix_timestamp(end_time) - remainder
            self.lbl_search.value = 'Querying PM for FGC entries on {}'.format(Time.to_string(start_time_temp))
            source_timestamp_fgc += PmDbRequest.find_events(metadata_fgc_rqd['source'], metadata_fgc_rqd['system'],
                                                            metadata_fgc_rqd['className'], t_start=start_time_temp,
                                                            duration=[(remainder, 'ns')])

        return source_timestamp_fgc

    def _keep_fgc_rqd_with_qps_and_rqf(self, circuit_type, circuit_names, source_timestamp_fgc):
        # Check if an RQD FGC timestamp has
        # # a QPS discharge around
        # # an RQF FGC timestamp
        source_timestamp_fgc_with_qps = []
        metadata_fgc_rqf = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_names[1], 'PC',
                                                                               'PM')
        metadata_qds = SignalMetadata.get_circuit_signal_database_metadata(circuit_type, circuit_names[0], 'QDS', 'PM')

        for index, elem in enumerate(source_timestamp_fgc):
            source_fgc, timestamp_fgc = elem
            # Update GUI
            self.lbl_search.value = 'Checking if there are QPS PM around an FGC timestamp {}'.format(
                Time.to_string(elem[1]))
            self.progress_search.description = f'FGC: {index + 1}/{len(source_timestamp_fgc)}'
            self.progress_search.value = index + 1

            # Check if there is at least one QPS timestamp around FGC timestamp
            source_timestamp_qds = PmDbRequest.find_events('*', metadata_qds['system'], metadata_qds['className'],
                                                           t_start=timestamp_fgc,
                                                           duration=[(60, 's'), (300, 's')])

            source_timestamp_qds_filtered = check_quench_presence_around_fqc_timestamp(circuit_type, circuit_names[0],
                                                                                       source_timestamp_qds)

            # Check if there is a corresponding RQF timestamp
            source_timestamp_fgc_rqf = PmDbRequest.find_events(metadata_fgc_rqf['source'], metadata_fgc_rqf['system'],
                                                               metadata_fgc_rqf['className'], t_start=timestamp_fgc,
                                                               duration=[(20, 'ms'), (20, 'ms')])
            # The FGC frequency is 50 Hz so we must allow a 20 ms time window (Zinur 30.03.2020)
            if source_timestamp_qds_filtered and source_timestamp_fgc_rqf:
                source_timestamp_fgc_with_qps.append(
                    [circuit_names[0], timestamp_fgc, circuit_names[1], source_timestamp_fgc_rqf[0][1]])

        self.lbl_search.value = 'Query completed.'

        return source_timestamp_fgc_with_qps
