import unittest
import pandas as pd
import matplotlib.pyplot as plt

from lhcsmapi.analysis.busbar.BusbarResistanceAnalysis import find_plateau_start_and_end, \
    find_start_end_of_the_longest_ramp_up
from test.resources.read_csv import read_csv


class TestBusbarResistanceAnalysis(unittest.TestCase):

    def test_find_plateau_start_and_end(self):
        # arrange
        i_meas_raw_df = read_csv('resources/busbar', 'I_MEAS')

        # act
        plateau_start_act, plateau_end_act = find_plateau_start_and_end(i_meas_raw_df, i_meas_threshold=50)

        ax = i_meas_raw_df.plot()
        for ps, pe in zip(plateau_start_act, plateau_end_act):
            ax.axvspan(xmin=ps, xmax=pe, facecolor='xkcd:green')
        plt.show()

        # assert
        plateau_start_ref = [1523190227160000000, 1523190778160000000, 1523193354660000000]
        plateau_end_ref = [1523190645160000000, 1523192221160000000, 1523195095160000000]

        self.assertListEqual(plateau_start_ref, plateau_start_act)
        self.assertListEqual(plateau_end_ref, plateau_end_act)

    def test_find_start_end_of_the_longest_ramp_up(self):
        # arrange
        i_meas_raw_df = read_csv('resources/busbar', 'I_MEAS')

        # act
        plateau_start, plateau_end = find_plateau_start_and_end(i_meas_raw_df, i_meas_threshold=50)

        ramp_up_start_act, ramp_up_end_act = find_start_end_of_the_longest_ramp_up(plateau_start, plateau_end)

        ax = i_meas_raw_df.plot()
        for ps, pe in zip(plateau_start, plateau_end):
            ax.axvspan(xmin=ps, xmax=pe, facecolor='xkcd:green')

        ax.axvspan(xmin=ramp_up_start_act, xmax=ramp_up_end_act, facecolor='xkcd:yellowgreen')
        plt.show()

        # assert
        ramp_up_start_ref = 1523192221160000000
        ramp_up_end_ref = 1523193354660000000

        self.assertEqual(ramp_up_start_ref, ramp_up_start_act)
        self.assertEqual(ramp_up_end_ref, ramp_up_end_act)


