import unittest
import pandas as pd
import numpy as np

from lhcsmapi.analysis.R600ACircuitAnalysis import _600ACircuitAnalysis


class Test600ACircuitQuery(unittest.TestCase):

    # WITH/WITHOUT EE

    def test_create_mp3_results_table(self):
        # arrange
        circuit_type = '600A'
        is_automatic = True
        results_table = pd.DataFrame({'Circuit Name': {0: 'ROD.A67B1'}, 'Family': {0: 'ROD'}, 'Period': {0: np.nan},
                                      'Date': {0: '2017-04-27'}, 'Time': {0: '00:36:16.040'},
                                      'flagged in APEX': {0: np.nan}, 'Quench Current': {0: 185.6},
                                      'I_MEAS MIITs': {0: 0.018}, 'Reason': {0: np.nan}, 'Pivot': {0: np.nan},
                                      'Quench Origin': {0: np.nan}, 'Ramp Rate': {0: -0.7},
                                      'Plateau Duration': {0: np.nan}, 'dUres/dt': {0: 4.422476047619048},
                                      'EE max voltage': {0: 113.57}, 'Earth Current': {0: 0.012},
                                      'Coupling': {0: np.nan}, 'Comment': {0: np.nan}, 'performed_by': {0: 'mmacieje'},
                                      'Reference Doc': {0: np.nan}, 'lhcsmapi_version': {0: '1.3.226'},
                                      'timestamp_sync': {0: 1493246176040000000},
                                      'timestamp_pic': {0: 1493246176022000000},
                                      'timestamp_ee': {0: 1493246176019000000},
                                      'timestamp_qds_a': {0: 1493246175924000000},
                                      'ROD.A67B1_LEADS_U_HTS': {0: 'Within reference'},
                                      'ROD.A67B1_LEADS_U_RES': {0: 'Within reference'}})
        analysis_rcd = _600ACircuitAnalysis(circuit_type, results_table, is_automatic=is_automatic)

        # act
        mp3_act = analysis_rcd.create_mp3_results_table()

        # assert
        mp3_ref = pd.DataFrame({'Circuit Name': {0: 'ROD.A67B1'}, 'Family': {0: 'ROD'}, 'Period': {0: np.nan},
                                'Date': {0: '2017-04-27'}, 'Time': {0: '00:36:16.040'}, 'flagged in APEX': {0: np.nan},
                                'Quench Current': {0: 185.6}, 'DCCT MIITs': {0: 0.018}, 'Reason': {0: np.nan},
                                'Pivot': {0: np.nan}, 'Quench Origin': {0: np.nan}, 'Ramp Rate': {0: -0.7},
                                'Plateau Duration': {0: np.nan}, 'dUres/dt': {0: 4.422476047619048},
                                'EE max voltage': {0: 113.57}, 'Earth Current': {0: 0.012}, 'Coupling': {0: np.nan},
                                'Comment': {0: np.nan}, 'Person who analyzed': {0: 'mmacieje'},
                                'Reference Doc': {0: np.nan}})

        mp3_act.fillna(np.nan, inplace=True)
        pd.testing.assert_frame_equal(mp3_ref, mp3_act)

    def test_create_mp3_results_table_rcdo(self):
        # arrange
        circuit_type = '600A'
        is_automatic = True
        results_table = pd.DataFrame({'Circuit Name': {0: 'RCD/O.A12B2'}, 'Family': {0: 'RCD'}, 'Period': {0: np.nan},
                                      'Date': {0: '2017-04-25'}, 'Time': {0: '18:55:03.160'},
                                      'flagged in APEX': {0: np.nan}, 'I_Q_RCO': {0: -105.0}, 'I_Q_RCD': {0: 349.7},
                                      'I_MEAS MIITs RCO': {0: 0.0}, 'I_MEAS MIITs RCD': {0: 0.004},
                                      'Reason RCD': {0: np.nan}, 'Reason RCO': {0: np.nan},
                                      'Quench Origin RCD': {0: np.nan}, 'Quench Origin RCO': {0: np.nan},
                                      'Ramp Rate RCO': {0: 0}, 'Ramp Rate RCD': {0: 1.5},
                                      'Plateau Duration': {0: 148.72}, 'dUres/dt RCO': {0: -97.546145},
                                      'dUres/dt RCD': {0: 6.9692185625}, 'Comment': {0: np.nan},
                                      'performed by': {0: np.nan}, 'Reference Doc': {0: np.nan},
                                      'performed_by': {0: ''}, 'lhcsmapi_version': {0: '1.3.226'},
                                      'timestamp_fgc_rcd': {0: 1493139303220000000},
                                      'timestamp_fgc_rco': {0: 1493139303160000000},
                                      'timestamp_pic_rcd': {0: 1493139303213000000},
                                      'timestamp_pic_rco': {0: 1493139303142000000},
                                      'timestamp_qds_a_rcd': {0: 1493139303215000000},
                                      'timestamp_qds_a_rco': {0: 1493139303146000000},
                                      'Earth Current RCD': {0: -0.062}, 'Earth Current RCO': {0: 0.008},
                                      'RCD.A12B2_LEADS_U_HTS': {0: 'Within reference'},
                                      'RCD.A12B2_LEADS_U_RES': {0: 'Within reference'},
                                      'RCO.A12B2_LEADS_U_HTS': {0: 'Within reference'},
                                      'RCO.A12B2_LEADS_U_RES': {0: 'Within reference'}})
        analysis_rcd = _600ACircuitAnalysis(circuit_type, results_table, is_automatic=is_automatic)

        # act
        mp3_act = analysis_rcd.create_mp3_results_table_rcdo()

        # assert
        mp3_ref = pd.DataFrame({'Circuit Name': {0: 'RCD/O.A12B2'}, 'Family': {0: 'RCD'}, 'Period': {0: np.nan},
                                'Date': {0: '2017-04-25'}, 'Time': {0: '18:55:03.160'}, 'flagged in APEX': {0: np.nan},
                                'I_Q_RCO': {0: -105.0}, 'I_Q_RCD': {0: 349.7}, 'I_MEAS MIITs RCO': {0: 0.0},
                                'I_MEAS MIITs RCD': {0: 0.004}, 'Reason': {0: np.nan}, 'Quench Origin': {0: np.nan},
                                'Ramp Rate RCO': {0: 0}, 'Ramp Rate RCD': {0: 1.5}, 'Plateau Duration': {0: 148.72},
                                'dUres/dt RCO': {0: -97.546145}, 'dUres/dt RCD': {0: 6.9692185625},
                                'Comment': {0: np.nan}, 'Person who analyzed': {0: ''}, 'Reference Doc': {0: np.nan}})

        mp3_act.fillna(np.nan, inplace=True)
        pd.testing.assert_frame_equal(mp3_ref, mp3_act)

    def test_create_mp3_results_table_rcbxhv(self):
        # arrange
        circuit_type = '600A'
        is_automatic = True
        results_table = pd.DataFrame({'Circuit Name': {0: 'RCBXV1.R1'}, 'Family': {0: 'MCBX'},
                                      'Period': {0: np.nan}, 'Date': {0: '2017-04-26'},
                                      'Time': {0: '17:07:49.220'}, 'flagged in APEX': {0: np.nan},
                                      'I_Q_H': {0: 134.0}, 'I_Q_V': {0: 387.6}, 'radius': {0: np.nan},
                                      'I_MEAS MIITs H': {0: 0.018}, 'I_MEAS MIITs V': {0: 0.084},
                                      'Reason H': {0: np.nan}, 'Reason V': {0: np.nan},
                                      'Quench Origin H': {0: np.nan}, 'Quench Origin V': {0: np.nan},
                                      'Ramp Rate H': {0: 2.2}, 'Ramp Rate V': {0: -0.7},
                                      'Plateau Duration': {0: np.nan},
                                      'dUres/dt H': {0: 87.70904875000001}, 'dUres/dt V': {0: np.nan},
                                      'Comment': {0: np.nan}, 'performed by': {0: np.nan},
                                      'Reference Doc': {0: np.nan}, 'performed_by': {0: ''},
                                      'lhcsmapi_version': {0: '1.3.226'},
                                      'timestamp_fgc_rcbxh': {0: 1493219269240000000},
                                      'timestamp_fgc_rcbxv': {0: 1493219269220000000},
                                      'timestamp_pic_rcbxh': {0: 1493219269239000000},
                                      'timestamp_pic_rcbxv': {0: 1493219269212000000},
                                      'timestamp_qds_a_rcbxh': {0: 1493219269213000000},
                                      'Earth Current H': {0: 0.004}, 'Earth Current V': {0: -0.006},
                                      'RCBXH1.R1_LEADS_U_HTS': {0: 'Within reference'},
                                      'RCBXH1.R1_LEADS_U_RES': {0: 'Within reference'},
                                      'RCBXV1.R1_LEADS_U_HTS': {0: 'Within reference'},
                                      'RCBXV1.R1_LEADS_U_RES': {0: 'Outside of reference'}})
        analysis_rcbxh = _600ACircuitAnalysis(circuit_type, results_table, is_automatic=is_automatic)

        # act
        mp3_act = analysis_rcbxh.create_mp3_results_table_rcbxhv()

        # assert
        mp3_ref = pd.DataFrame({'Circuit Name': {0: 'RCBXV1.R1'}, 'Family': {0: 'MCBX'}, 'Period': {0: np.nan},
                                'Date': {0: '2017-04-26'}, 'Time': {0: '17:07:49.220'}, 'flagged in APEX': {0: np.nan},
                                'I_Q_H': {0: 134.0}, 'I_Q_V': {0: 387.6}, 'radius': {0: 410.1094488060474},
                                'I_MEAS MIITs H': {0: 0.018}, 'I_MEAS MIITs V': {0: 0.084}, 'Reason': {0: np.nan},
                                'Quench Origin': {0: np.nan}, 'Ramp Rate H': {0: 2.2}, 'Ramp Rate V': {0: -0.7},
                                'Plateau Duration': {0: np.nan}, 'dUres/dt H': {0: 87.70904875000001},
                                'dUres/dt V': {0: np.nan}, 'Comment': {0: np.nan}, 'Person who analyzed': {0: ''},
                                'Reference Doc': {0: np.nan}})

        mp3_act.fillna(np.nan, inplace=True)
        pd.testing.assert_frame_equal(mp3_ref, mp3_act)
