import unittest
import pandas as pd
import numpy as np

from lhcsmapi.analysis.R600ACircuitQuery import R600ACircuitQuery


class Test600ACircuitQuery(unittest.TestCase):

    def test_create_report_analysis_template(self):
        # arrange
        query = R600ACircuitQuery('600A', 'RCBXH.R1')

        # act
        results_act = query.create_report_analysis_template(1493219269220000000)
        results_act['lhcsmapi_version'] = '1.3.226'

        # assert
        results_ref = pd.DataFrame({'Circuit Name': {0: 'RCBXH.R1'}, 'Family': {0: 'RCBXH'}, 'Period': {0: np.nan},
                                    'Date': {0: '2017-04-26'}, 'Time': {0: '17:07:49.220'},
                                    'flagged in APEX': {0: np.nan}, 'Quench Current': {0: np.nan},
                                    'I_MEAS MIITs': {0: np.nan}, 'Reason': {0: np.nan}, 'Pivot': {0: np.nan},
                                    'Quench Origin': {0: np.nan}, 'Ramp Rate': {0: np.nan},
                                    'Plateau Duration': {0: np.nan}, 'dUres/dt': {0: np.nan},
                                    'EE max voltage': {0: np.nan}, 'Earth Current': {0: np.nan},
                                    'Coupling': {0: np.nan}, 'Comment': {0: np.nan}, 'performed_by': {0: ''},
                                    'Reference Doc': {0: np.nan}, 'lhcsmapi_version': {0: '1.3.226'}})

        results_act.fillna(np.nan, inplace=True)
        pd.testing.assert_frame_equal(results_ref, results_act)

    def test_create_report_analysis_template_rcd(self):
        # arrange
        timestamp_fgc_rcd = 1493139303220000000
        timestamp_fgc_rco = 1493139303160000000
        circuit_names = ['RCD.A12B2', 'RCO.A12B2']
        query = R600ACircuitQuery('600A', circuit_names[0])

        # act
        results_act = query.create_report_analysis_template_rcd(timestamp_fgc_rcd, timestamp_fgc_rco, circuit_names)
        results_act['lhcsmapi_version'] = '1.3.226'

        # assert
        results_ref = pd.DataFrame({'Circuit Name': {0: 'RCD/O.A12B2'},  'Family': {0: 'RCD'}, 'Period': {0: np.nan},
                                    'Date': {0: '2017-04-25'}, 'Time': {0: '18:55:03.160'},
                                    'flagged in APEX': {0: np.nan}, 'I_Q_RCO': {0: np.nan}, 'I_Q_RCD': {0: np.nan},
                                    'I_MEAS MIITs RCO': {0: np.nan}, 'I_MEAS MIITs RCD': {0: np.nan},
                                    'Reason RCD': {0: np.nan}, 'Reason RCO': {0: np.nan},
                                    'Quench Origin RCD': {0: np.nan}, 'Quench Origin RCO': {0: np.nan},
                                    'Ramp Rate RCO': {0: np.nan}, 'Ramp Rate RCD': {0: np.nan},
                                    'Plateau Duration': {0: np.nan}, 'dUres/dt RCO': {0: np.nan},
                                    'dUres/dt RCD': {0: np.nan}, 'Comment': {0: np.nan}, 'performed_by': {0: ''},
                                    'Reference Doc': {0: np.nan}, 'lhcsmapi_version': {0: '1.3.226'}})

        results_act.fillna(np.nan, inplace=True)
        pd.testing.assert_frame_equal(results_ref, results_act)

    def test_create_report_analysis_template_rcbx(self):
        # arrange
        timestamp_fgc_rcd = 1493219269240000000
        timestamp_fgc_rco = 1493219269220000000
        circuit_names = ['RCBXH1.R1', 'RCBXV1.R1']
        query = R600ACircuitQuery('600A', circuit_names[0])

        # act
        results_act = query.create_report_analysis_template_rcbx(timestamp_fgc_rcd, timestamp_fgc_rco, circuit_names)
        results_act['lhcsmapi_version'] = '1.3.226'

        # assert
        results_ref = pd.DataFrame({'Circuit Name': {0: 'RCBXV1.R1'}, 'Family': {0: 'MCBX'}, 'Period': {0: np.nan},
                                    'Date': {0: '2017-04-26'}, 'Time': {0: '17:07:49.220'},
                                    'flagged in APEX': {0: np.nan}, 'I_Q_H': {0: np.nan}, 'I_Q_V': {0: np.nan},
                                    'radius': {0: np.nan}, 'I_MEAS MIITs H': {0: np.nan}, 'I_MEAS MIITs V': {0: np.nan},
                                    'Reason H': {0: np.nan}, 'Reason V': {0: np.nan}, 'Quench Origin H': {0: np.nan},
                                    'Quench Origin V': {0: np.nan}, 'Ramp Rate H': {0: np.nan},
                                    'Ramp Rate V': {0: np.nan}, 'Plateau Duration': {0: np.nan},
                                    'dUres/dt H': {0: np.nan}, 'dUres/dt V': {0: np.nan}, 'Comment': {0: np.nan},
                                    'performed_by': {0: ''}, 'Reference Doc': {0: np.nan},
                                    'lhcsmapi_version': {0: '1.3.226'}})

        results_act.fillna(np.nan, inplace=True)
        pd.testing.assert_frame_equal(results_ref, results_act)
