import unittest
import pandas as pd

from lhcsmapi.analysis.RbCircuitAnalysis import RbCircuitAnalysis, analyze_qds_trigger
from lhcsmapi.pyedsl.AssertionBuilder import AssertionBuilder
from test.test_analysis.hwc.read_csv import read_csv


class RbCircuitAnalysisTest(unittest.TestCase):

    def setUp(self):
        self.circuit_type = 'RB'
        self.circuit_name = 'RB.A12'
        self.timestamp_fgc = 1544631694840000000
        self.timestamp_fgc_ref = 1521298828680000000
        self.timestamp_pic = 1544631694800000000
        self.timestamp_ee_odd = 1544631694896000000
        self.timestamp_ee_even = 1544631695395000000
        report_analysis = read_csv('rb', 'report_analysis')
        report_analysis = report_analysis[report_analysis.index == 0]
        self.rb_analysis = RbCircuitAnalysis(self.circuit_type, report_analysis, True)

    def test_analyze_i_meas_pc(self):
        i_meas_df = read_csv('rb', 'i_meas_df')
        i_meas_ref_df = read_csv('rb', 'i_meas_ref_df')

        self.rb_analysis.analyze_i_meas_pc(self.circuit_name, self.timestamp_fgc, self.timestamp_fgc_ref,
                                           self.timestamp_pic, i_meas_df, i_meas_ref_df)

    def test_analyze_i_earth_pc(self):
        i_a_df = read_csv('rb', 'i_a_df')
        i_earth_df = read_csv('rb', 'i_earth_df')
        i_earth_ref_df = read_csv('rb', 'i_earth_ref_df')

        self.rb_analysis.analyze_i_earth_pc(self.circuit_type, self.circuit_name, self.timestamp_fgc, i_a_df,
                                            i_earth_df, i_earth_ref_df)

    def test_analyze_i_earth_pcnt_pc(self):
        i_meas_df = read_csv('rb', 'i_meas_df')
        i_earth_pcnt_df = read_csv('rb', 'i_earth_pcnt_df')
        i_earth_pcnt_ref_df = read_csv('rb', 'i_earth_pcnt_ref_df')

        self.rb_analysis.analyze_i_earth_pc(self.circuit_type, self.circuit_name, self.timestamp_fgc, i_meas_df,
                                            i_earth_pcnt_df, i_earth_pcnt_ref_df)

    def test_analyze_char_time_u_dump_res_ee(self):
        u_dump_res_odd_df = read_csv('rb', 'u_dump_res_odd_df')
        i_meas_df = read_csv('rb', 'i_meas_df')

        self.rb_analysis.analyze_char_time_u_dump_res_ee(self.circuit_name, self.timestamp_fgc, u_dump_res_odd_df,
                                                         i_meas_df)

    def test_analyze_delay_time_u_dump_res_ee_rqf(self):
        i_a_df = read_csv('rb', 'i_a_df')
        i_ref_df = read_csv('rb', 'i_ref_df')
        u_dump_res_odd_df = read_csv('rb', 'u_dump_res_odd_df')
        u_dump_res_even_df = read_csv('rb', 'u_dump_res_even_df')

        self.rb_analysis.analyze_delay_time_u_dump_res_ee(self.circuit_name, self.timestamp_fgc, self.timestamp_pic,
                                                          [self.timestamp_ee_odd, self.timestamp_ee_even], i_a_df,
                                                          i_ref_df, [u_dump_res_odd_df, u_dump_res_even_df])

    def test_temperature_ee_odd(self):

        t_res_body_odd_dfs = [read_csv('rb', 't_res_body_1_odd_0_df'), read_csv('rb', 't_res_body_2_odd_0_df'),
                              read_csv('rb', 't_res_body_3_odd_0_df'), read_csv('rb', 't_res_body_1_odd_1_df'),
                              read_csv('rb', 't_res_body_2_odd_1_df'), read_csv('rb', 't_res_body_3_odd_1_df')]

        t_res_body_odd_ref_dfs = [read_csv('rb', 't_res_body_1_odd_0_ref_df'), read_csv('rb', 't_res_body_2_odd_0_ref_df'),
                                  read_csv('rb', 't_res_body_3_odd_0_ref_df'), read_csv('rb', 't_res_body_1_odd_1_ref_df'),
                                  read_csv('rb', 't_res_body_2_odd_1_ref_df'), read_csv('rb', 't_res_body_3_odd_1_ref_df')]

        AssertionBuilder().with_signal(t_res_body_odd_dfs) \
            .compare_to_reference(signal_ref_dfs=t_res_body_odd_ref_dfs, abs_margin=25, scaling=1)
        
    def test_temperature_ee_even(self):

        t_res_body_even_dfs = [read_csv('rb', 't_res_body_1_even_0_df'), read_csv('rb', 't_res_body_2_even_0_df'),
                               read_csv('rb', 't_res_body_3_even_0_df'), read_csv('rb', 't_res_body_1_even_1_df'),
                               read_csv('rb', 't_res_body_2_even_1_df'), read_csv('rb', 't_res_body_3_even_1_df')]

        t_res_body_even_ref_dfs = [read_csv('rb', 't_res_body_1_even_0_ref_df'), read_csv('rb', 't_res_body_2_even_0_ref_df'),
                                   read_csv('rb', 't_res_body_3_even_0_ref_df'), read_csv('rb', 't_res_body_1_even_1_ref_df'),
                                   read_csv('rb', 't_res_body_2_even_1_ref_df'), read_csv('rb', 't_res_body_3_even_1_ref_df')]

        AssertionBuilder().with_signal(t_res_body_even_dfs) \
            .compare_to_reference(signal_ref_dfs=t_res_body_even_ref_dfs, abs_margin=25, scaling=1)

    def test_analyze_qh(self):

        u_hds_dfs = [[read_csv('rb', 'u_hds_1_df'), read_csv('rb', 'u_hds_2_df'), read_csv('rb', 'u_hds_3_df'), read_csv('rb', 'u_hds_4_df')]]
        i_hds_dfs = [[read_csv('rb', 'i_hds_1_df'), read_csv('rb', 'i_hds_2_df'), read_csv('rb', 'i_hds_3_df'), read_csv('rb', 'i_hds_4_df')]]

        u_hds_ref_dfs = [[read_csv('rb', 'u_hds_1_ref_df'), read_csv('rb', 'u_hds_2_ref_df'),
                          read_csv('rb', 'u_hds_3_ref_df'), read_csv('rb', 'u_hds_4_ref_df')]]
        i_hds_ref_dfs = [[read_csv('rb', 'i_hds_1_ref_df'), read_csv('rb', 'i_hds_2_ref_df'),
                          read_csv('rb', 'i_hds_3_ref_df'), read_csv('rb', 'i_hds_4_ref_df')]]

        self.rb_analysis.analyze_qh(u_hds_dfs, i_hds_dfs, u_hds_ref_dfs, i_hds_ref_dfs)

    def test_analyze_qds_trigger(self):
        # arrange
        u_qs0_df = read_csv('rb', 'u_qs0_0_df')
        st_nqd0_df = read_csv('rb', 'st_nqd0_0_df')
        st_magnet_ok_df = read_csv('rb', 'st_magnet_ok_0_df')

        # act
        report_qds_trigger_act = analyze_qds_trigger(u_qs0_df, st_magnet_ok_df, st_nqd0_df)

        # assert
        report_qds_trigger_ref = pd.DataFrame({'t_st_magnet_ok [s]': {0: -0.016},
                                               't_st_nqd0 [s]': {0: -0.026},
                                               'u_st_nqd0 [V]': {0: 0.0019524792},
                                               't_start_quench [s]': {0: -0.038},
                                               'du_dt [V/s]': {0: -7.914204933333333},
                                               't_delay_qh_trigger [s]': {0: 0.016},
                                               'analysis_comment': {0: ''}})

        pd.testing.assert_frame_equal(report_qds_trigger_ref, report_qds_trigger_act)

    #    def test_mocking(self):
    #     from unittest.mock import patch
    #     i_meas_df = read_csv('i_meas_df')
    #
    #     with self.subTest(name=apiKey):
    #         with patch.object(behavior, 'read') as mock_read:
    #             mock_read.return_value = dfReturned
    #
    #     i_meas_df = QueryBuilder().with_pm() \
    #         .with_timestamp(1426220469520000000) \
    #         .with_query_parameters(system='FGC', source='RPTE.UA47.RB.A45', className='51_self_pmd',
    #                                signal='STATUS.I_MEAS') \
    #         .signal_query().dfs[0]
    #
    #
