import unittest

import pandas as pd
import numpy as np

from lhcsmapi.analysis.RqCircuitAnalysis import RqCircuitAnalysis
from lhcsmapi.pyedsl.AssertionBuilder import AssertionBuilder

from test.test_analysis.hwc.read_csv import read_csv


class RqCircuitAnalysisTest(unittest.TestCase):

    def setUp(self):
        self.circuit_type = 'RQ'
        self.circuit_names = ['RQD.A12', 'RQF.A12']
        self.timestamp_fgc_rqd = 1544622149620000000
        self.timestamp_fgc_rqf = 1544622149620000000
        self.timestamp_fgc_ref_rqd = 1521298828680000000
        self.timestamp_fgc_ref_rqf = 1521298828680000000
        self.timestamp_pic_rqd = 1544622149611000000
        self.timestamp_pic_rqf = 1544622149611000000
        self.source_timestamp_ee_rqd_df = pd.DataFrame({'source': {0: 'UA23.RQD.A12', 1: 'UA23.RQD.A12'},
                                                        'timestamp': {0: 1544622149701000000, 1: 1544622569004000000}})
        self.timestamp_ee_rqd = 1544622149701000000
        self.source_timestamp_ee_rqf_df = pd.DataFrame({'source': {0: 'UA23.RQF.A12', 1: 'UA23.RQF.A12'},
                                                        'timestamp': {0: 1544622149701000000, 1: 1544622571604000000}})
        self.timestamp_ee_rqf = 1544622149701000000
        self.timestamp_ee_ref_rqd = 1521370713675000000
        self.timestamp_ee_ref_rqf = 1521370713675000000

        self.source_timestamp_ee_rqd_ref_df = pd.DataFrame({'source': {0: 'UA23.RQD.A12', 1: 'UA23.RQD.A12'},
                                                            'timestamp': {0: 1521370713675000000,
                                                                          1: 1521371135404000000}})
        self.source_timestamp_ee_rqf_ref_df = pd.DataFrame({'source': {0: 'UA23.RQF.A12', 1: 'UA23.RQF.A12'},
                                                            'timestamp': {0: 1521370713675000000,
                                                                          1: 1521371141604000000}})
        self.source_timestamp_qds_rq_df = pd.DataFrame({'source': {0: '16L2'},
                                                        'timestamp': {0: 1544622149598000000}})
        self.source_timestamp_nqps_rqd_df = pd.DataFrame({'source': {0: 'B17L2'},
                                                          'timestamp': {0: 1544622149613000000}})

        self.source_timestamp_leads_rqd_df = pd.DataFrame()
        self.source_timestamp_leads_rqf_df = pd.DataFrame()

        report_analysis = pd.DataFrame(read_csv('rq', 'report_analysis'))
        self.rq_analysis = RqCircuitAnalysis(self.circuit_type, report_analysis, is_automatic=True)

    def test_create_timestamp_report(self):
        # arrange
        timestamp_dct = {'FGC_RQD': self.timestamp_fgc_rqd, 'FGC_RQF': self.timestamp_fgc_rqf,
                         'PIC_RQD': self.timestamp_pic_rqd, 'PIC_RQF': self.timestamp_pic_rqf,
                         'EE_RQD': self.source_timestamp_ee_rqd_df, 'EE_RQF': self.source_timestamp_ee_rqf_df,
                         'iQPS': self.source_timestamp_qds_rq_df, 'nQPS': self.source_timestamp_nqps_rqd_df,
                         'LEADS_RQD': self.source_timestamp_leads_rqd_df,
                         'LEADS_RQF': self.source_timestamp_leads_rqf_df}

        # act
        timestamp_report_act = self.rq_analysis.create_timestamp_table(timestamp_dct)

        timestamp_report_ref = pd.DataFrame({'Source': {0: 'iQPS',
                                                        1: 'PIC_RQD',
                                                        2: 'PIC_RQF',
                                                        3: 'nQPS',
                                                        4: 'FGC_RQD',
                                                        5: 'FGC_RQF',
                                                        6: 'EE_RQD',
                                                        7: 'EE_RQF'},
                                             'Timestamp': {0: 1544622149598000000,
                                                           1: 1544622149611000000,
                                                           2: 1544622149611000000,
                                                           3: 1544622149613000000,
                                                           4: 1544622149620000000,
                                                           5: 1544622149620000000,
                                                           6: 1544622149701000000,
                                                           7: 1544622149701000000},
                                             'Date and time': {0: '2018-12-12 14:42:29.598',
                                                               1: '2018-12-12 14:42:29.611',
                                                               2: '2018-12-12 14:42:29.611',
                                                               3: '2018-12-12 14:42:29.613',
                                                               4: '2018-12-12 14:42:29.620',
                                                               5: '2018-12-12 14:42:29.620',
                                                               6: '2018-12-12 14:42:29.701',
                                                               7: '2018-12-12 14:42:29.701'}})

        # assert
        pd.testing.assert_frame_equal(timestamp_report_ref, timestamp_report_act)

    def test_analyze_i_meas_rqd_pc(self):
        i_meas_rqd_df = read_csv('rq', 'i_meas_rqd_df')
        i_meas_ref_rqd_df = read_csv('rq', 'i_meas_ref_rqd_df')

        self.rq_analysis.analyze_i_meas_pc(self.circuit_names[0], self.timestamp_fgc_rqd,
                                                               self.timestamp_fgc_ref_rqd, self.timestamp_pic_rqd,
                                                               i_meas_rqd_df, i_meas_ref_rqd_df)

    def test_analyze_i_meas_rqf_pc(self):
        i_meas_rqf_df = read_csv('rq', 'i_meas_rqf_df')
        i_meas_ref_rqf_df = read_csv('rq', 'i_meas_ref_rqf_df')

        self.rq_analysis.analyze_i_meas_pc(self.circuit_names[1], self.timestamp_fgc_rqf, self.timestamp_fgc_ref_rqf,
                                           self.timestamp_pic_rqf, i_meas_rqf_df, i_meas_ref_rqf_df)

    def test_analyze_i_earth_pc_rqd(self):
        i_a_rqd_df = read_csv('rq', 'i_a_rqd_df')
        i_earth_rqd_df = read_csv('rq', 'i_earth_rqd_df')
        i_earth_rqd_ref_df = read_csv('rq', 'i_earth_rqd_ref_df')

        self.rq_analysis.analyze_i_earth_pc(self.circuit_type, self.circuit_names[0], self.timestamp_fgc_rqd,
                                            i_a_rqd_df, i_earth_rqd_df, i_earth_rqd_ref_df)

    def test_analyze_i_earth_pc_rqf(self):
        i_a_rqf_df = read_csv('rq', 'i_a_rqf_df')
        i_earth_rqf_df = read_csv('rq', 'i_earth_rqf_df')
        i_earth_rqf_ref_df = read_csv('rq', 'i_earth_rqf_ref_df')

        self.rq_analysis.analyze_i_earth_pc(self.circuit_type, self.circuit_names[0], self.timestamp_fgc_rqd,
                                            i_a_rqf_df, i_earth_rqf_df, i_earth_rqf_ref_df)

    def test_analyze_char_time_u_dump_res_rqd(self):
        u_dump_res_rqd_df = read_csv('rq', 'u_dump_res_rqd_df')
        i_meas_rqd_df = read_csv('rq', 'i_meas_rqd_df')

        self.rq_analysis.analyze_char_time_u_dump_res_ee(self.circuit_names[0], self.timestamp_fgc_rqd,
                                                         u_dump_res_rqd_df, i_meas_rqd_df)

    def test_analyze_char_time_u_dump_res_rqf(self):
        u_dump_res_rqf_df = read_csv('rq', 'u_dump_res_rqf_df')
        i_meas_rqf_df = read_csv('rq', 'i_meas_rqf_df')

        self.rq_analysis.analyze_char_time_u_dump_res_ee(self.circuit_names[0], self.timestamp_fgc_rqd,
                                                         u_dump_res_rqf_df, i_meas_rqf_df)

    def test_analyze_delay_time_u_dump_res_ee_rqd(self):
        i_a_rqd_df = read_csv('rq', 'i_a_rqd_df')
        i_ref_rqd_df = read_csv('rq', 'i_ref_rqd_df')
        u_dump_res_rqd_df = read_csv('rq', 'u_dump_res_rqd_df')

        self.rq_analysis.analyze_delay_time_u_dump_res_ee(self.circuit_names[0], self.timestamp_fgc_rqd,
                                                          self.timestamp_pic_rqd, self.timestamp_ee_rqd,
                                                          i_a_rqd_df, i_ref_rqd_df, u_dump_res_rqd_df)

    def test_analyze_delay_time_u_dump_res_ee_rqf(self):
        i_a_rqf_df = read_csv('rq', 'i_a_rqf_df')
        i_ref_rqf_df = read_csv('rq', 'i_ref_rqf_df')
        u_dump_res_rqf_df = read_csv('rq', 'u_dump_res_rqf_df')

        self.rq_analysis.analyze_delay_time_u_dump_res_ee(self.circuit_names[0], self.timestamp_fgc_rqd,
                                                          self.timestamp_pic_rqd, self.timestamp_ee_rqd, i_a_rqf_df,
                                                          i_ref_rqf_df, u_dump_res_rqf_df)

    def test_temperature_ee_rqd(self):
        t_res_0_rqd_df = read_csv('rq', 't_res_0_rqd_df')
        t_res_1_rqd_df = read_csv('rq', 't_res_1_rqd_df')

        t_res_0_rqd_ref_df = read_csv('rq', 't_res_0_rqd_ref_df')
        t_res_1_rqd_ref_df = read_csv('rq', 't_res_1_rqd_ref_df')

        AssertionBuilder().with_signal([t_res_0_rqd_df, t_res_1_rqd_df]) \
            .compare_to_reference(signal_ref_dfs=[t_res_0_rqd_ref_df, t_res_1_rqd_ref_df], abs_margin=25, scaling=1)

    def test_temperature_ee_rqf(self):
        t_res_0_rqf_df = read_csv('rq', 't_res_0_rqf_df')
        t_res_1_rqf_df = read_csv('rq', 't_res_1_rqf_df')

        t_res_0_rqf_ref_df = read_csv('rq', 't_res_0_rqf_ref_df')
        t_res_1_rqf_ref_df = read_csv('rq', 't_res_1_rqf_ref_df')

        AssertionBuilder().with_signal([t_res_0_rqf_df, t_res_1_rqf_df]) \
            .compare_to_reference(signal_ref_dfs=[t_res_0_rqf_ref_df, t_res_1_rqf_ref_df], abs_margin=25, scaling=1)

    def test_analyze_qds(self):
        iqps_analog_dfs = [[read_csv('rq', 'u_qs0_ext_rq_df'), read_csv('rq', 'u_qs0_int_rq_df'),
                            read_csv('rq', 'u_1_ext_rq_df'), read_csv('rq', 'u_2_ext_rq_df'),
                            read_csv('rq', 'u_1_int_rq_df'), read_csv('rq', 'u_2_int_rq_df')]]

        iqps_digital_dfs = [[read_csv('rq', 'st_magnet_ok_ext_rq_df'), read_csv('rq', 'st_magnet_ok_int_rq_df'),
                             read_csv('rq', 'st_nqd0_ext_rq_df'), read_csv('rq', 'st_nqd0_int_rq_df')]]

        # due to a conflict with u_diode_rqd_df used for DIODE analysis, the signal is referred to as u_diode_rqd_pm_df
        u_nqps_rqd_dfs = [[read_csv('rq', 'u_diode_rqd_pm_df'), read_csv('rq', 'u_ref_rqd_df')]]
        # due to a conflict with u_diode_rqf_df used for DIODE analysis, the signal is referred to as u_diode_rqf_pm_df
        u_nqps_rqf_dfs = [[read_csv('rq', 'u_diode_rqf_pm_df'), read_csv('rq', 'u_ref_rqf_df')]]

        self.rq_analysis.analyze_qds(self.circuit_names, iqps_analog_dfs, iqps_digital_dfs, u_nqps_rqd_dfs,
                                     u_nqps_rqf_dfs)

    def test_analyze_qh(self):


        u_hds_rq_dfs = [[read_csv('rq', 'u_hds_1_rq_df'), read_csv('rq', 'u_hds_2_rq_df')]]
        u_hds_rq_ref_dfs = [[read_csv('rq', 'u_hds_1_rq_ref_df'), read_csv('rq', 'u_hds_2_rq_ref_df')]]

        self.rq_analysis.analyze_qh(self.circuit_type, u_hds_rq_dfs, u_hds_rq_ref_dfs)

    def test_analyze_diode_leads_rqd(self):

        # due to a conflict with i_a_rqd_df used for PC analysis, the signal is referred to as i_a_rqd_raw_df
        i_meas_u_diode_u_ref_rqd_pm_dfs = [[read_csv('rq', 'i_a_rqd_raw_df'), read_csv('rq', 'u_diode_a_rqd_df'),
                                            read_csv('rq', 'u_diode_b_rqd_df'), read_csv('rq', 'u_ref_rqd_df')]]

        i_meas_u_diode_rqd_nxcals_dfs = [[read_csv('rq', 'i_meas_diode_rqd_df'), read_csv('rq', 'u_diode_rqd_df')]]

        self.rq_analysis.analyze_diode_leads(self.circuit_names[0], i_meas_u_diode_u_ref_rqd_pm_dfs,
                                             i_meas_u_diode_rqd_nxcals_dfs)
        
    def test_analyze_diode_leads_rqf(self):
        # due to a conflict with i_a_rqf_df used for PC analysis, the signal is referred to as i_a_rqf_raw_df
        i_meas_u_diode_u_ref_rqf_pm_dfs = [[read_csv('rq', 'i_a_rqf_raw_df'), read_csv('rq', 'u_diode_a_rqf_df'),
                                            read_csv('rq', 'u_diode_b_rqf_df'), read_csv('rq', 'u_ref_rqf_df')]]

        i_meas_u_diode_rqf_nxcals_dfs = [[read_csv('rq', 'i_meas_diode_rqf_df'), read_csv('rq', 'u_diode_rqf_df')]]

        self.rq_analysis.analyze_diode_leads(self.circuit_names[0], i_meas_u_diode_u_ref_rqf_pm_dfs,
                                             i_meas_u_diode_rqf_nxcals_dfs)

    def test_analyze_leads_u_res(self):
        u_res_rqd_dfs = [read_csv('rq', 'u_res_1_rqd_df'), read_csv('rq', 'u_res_2_rqd_df')]
        u_res_rqf_dfs = [read_csv('rq', 'u_res_1_rqf_df'), read_csv('rq', 'u_res_2_rqf_df')]
        AssertionBuilder().with_signal(u_res_rqd_dfs + u_res_rqf_dfs) \
            .has_min_max_value(value_min=-0.1, value_max=0.1)
    
    def test_analyze_leads_u_hts(self):
        u_hts_rqd_dfs = [read_csv('rq', 'u_hts_1_rqd_df'), read_csv('rq', 'u_hts_2_rqd_df')]
        u_hts_rqf_dfs = [read_csv('rq', 'u_hts_1_rqf_df'), read_csv('rq', 'u_hts_2_rqf_df')]
        AssertionBuilder().with_signal(u_hts_rqd_dfs + u_hts_rqf_dfs) \
            .has_min_max_value(value_min=-0.003, value_max=0.003)

    def test_create_mp3_results_table(self):
        # arrange
        circuit_type = 'RQ'
        circuit_names = ['RQD.A12', 'RQF.A12']
        is_automatic = True
        results_table = read_csv('rq', 'report_analysis')
        rq_analysis = RqCircuitAnalysis(circuit_type, results_table, is_automatic=is_automatic)

        # act
        mp3_act = rq_analysis.create_mp3_results_table(circuit_names, self.timestamp_fgc_rqd)

        # assert
        mp3_ref = pd.DataFrame({'Circuit': {0: 'RQ.A12'}, 'Date': {0: '2018-12-12'},  'Time': {0: '14:42:29'},
                                'FGC PM file': {0: '20181212_144229.620_RPHE.UA23.RQx.A12'},
                                'iQPS PM file': {0: '20181212_144229.598_16L2'}, 'flagged in APEX': {0: np.nan},
                                'iQPS or nQPS': {0: np.nan}, 'I_q_D': {0: 11154}, 'I_q_F': {0: 11154},
                                'Type of Q': {0: np.nan}, 'Position': {0: '16L2'}, 'Magnet ID#1': {0: np.nan},
                                'Magnet ID#2': {0: np.nan}, 'SSS ID': {0: np.nan}, 'I_q_SM18': {0: np.nan},
                                'd_Iq': {0: np.nan},  'Origin': {0: np.nan}, 'dt_thres_INT': {0: np.nan},
                                'dt_thres_EXT': {0: np.nan},  'R_lead_init_RQF': {0: np.nan},
                                'R_lead_init_RQD': {0: np.nan}, 'R_lead_max_RQD': {0: np.nan},
                                'R_lead_max_RQF': {0: np.nan}, 't_lead_max_RQD': {0: np.nan},
                                't_lead_max_RQF': {0: np.nan}, 'Comments': {0: np.nan}})
        mp3_act.fillna(np.nan, inplace=True)
        mp3_ref.fillna(np.nan, inplace=True)
        pd.testing.assert_frame_equal(mp3_act, mp3_ref)







