import unittest
import pandas as pd
from pathlib import Path
import os

from lhcsmapi.dbsignal.ExpSignalAnalysis import ExpSignalAnalysis
from lhcsmapi.dbsignal.SignalUtilities import SignalUtilities


def read_qh_discharge_from_csv(path, timestamp_qds, magnet):
    file_path = os.path.join(str(path.parent), 'resources/dbsignal/{}_{}_I_HDS_U_HDS.csv'.format(timestamp_qds, magnet))
    qh_df = pd.read_csv(file_path, index_col=0)
    # Synchronize time to 0
    qh_sync_df = SignalUtilities.synchronize_df(qh_df, timestamp_qds)
    # Convert indices to seconds
    return SignalUtilities.convert_indices_to_sec(qh_sync_df)


class TestExponentialSignalAnalysis(unittest.TestCase):

    def setUp(self):
        self.path = Path(os.path.dirname(__file__))

    def tearDown(self):
        pass

    def test_get_characteristic_time_of_exponential_decay_A31R1(self):
        # Arrange
        timestamp_qds = 1544631694793000000
        magnet = 'A31R1'
        qh_df_A31R1 = read_qh_discharge_from_csv(self.path, timestamp_qds, magnet)

        # Act
        index_decay_start = ExpSignalAnalysis.get_window_with_given_mean_and_std(qh_df_A31R1['I_HDS_1'])
        t_0_act = qh_df_A31R1.index[index_decay_start]

        # Assert
        t_0_exp = 0.006181
        self.assertEqual(t_0_exp, t_0_act)

    def test_get_characteristic_time_of_exponential_decay_A10L7(self):
        # Arrange
        # Read another discharge for tests
        timestamp_qds = 1412844934913000000
        magnet = 'A10L7'
        qh_df_A10L7 = read_qh_discharge_from_csv(self.path, timestamp_qds, magnet)

        # Act
        index_decay_start = ExpSignalAnalysis.get_window_with_given_mean_and_std(qh_df_A10L7['I_HDS_1'])
        t_0_act = qh_df_A10L7.index[index_decay_start]

        # Assert
        t_0_exp = 0.00089
        self.assertEqual(t_0_exp, t_0_act)

    def test_get_characteristic_time_of_exponential_decay_B29L8(self):
        # Arrange
        # Read another discharge for tests
        timestamp_qds = 1544269623912000000
        magnet = 'B29L8'
        qh_df_B29L8 = read_qh_discharge_from_csv(self.path, timestamp_qds, magnet)

        # Act
        index_decay_start = ExpSignalAnalysis.get_window_with_given_mean_and_std(qh_df_B29L8['I_HDS_1'])
        t_0_act = qh_df_B29L8.index[index_decay_start]

        # Assert
        t_0_exp = 2e-05
        self.assertEqual(t_0_exp, t_0_act)



