import unittest
import numpy as np
import pandas as pd

from lhcsmapi.pyedsl.FeatureBuilder import FeatureBuilder


class TestFeatureBuilder(unittest.TestCase):

    def test_calculate_features_df(self):
        # arrange
        x = np.arange(100) * 0.001
        tau = 0.8
        df = pd.DataFrame(data=np.exp(-x / tau), index=x, columns=["Voltage_1"])

        # act
        function_names_to_apply = ["tau_charge", "tau_energy", "tau_lin_reg", "tau_exp_fit"]
        features = FeatureBuilder().with_signal(df).calculate_features(function_names_to_apply)

        # assert
        self.assertAlmostEqual(tau, features.loc[0, 'Voltage_1:tau_charge'], places=5)
        self.assertAlmostEqual(tau, features.loc[0, 'Voltage_1:tau_energy'], places=5)
        self.assertAlmostEqual(tau, features.loc[0, 'Voltage_1:tau_lin_reg'], places=5)
        self.assertAlmostEqual(tau, features.loc[0, 'Voltage_1:tau_exp_fit'], places=5)

    def test_calculate_features_dfs(self):
        # arrange
        x = np.arange(100) * 0.001
        tau_1 = 0.8
        df_1 = pd.DataFrame(data=np.exp(-x / tau_1), index=x, columns=["Voltage_1"])
        tau_2 = 0.4
        df_2 = pd.DataFrame(data=np.exp(-x / tau_2), index=x, columns=["Voltage_2"])

        # act
        function_names_to_apply = ["tau_charge", "tau_energy", "tau_lin_reg", "tau_exp_fit"]
        features = FeatureBuilder().with_signal(df_1, df_2).calculate_features(function_names_to_apply)

        # assert
        self.assertAlmostEqual(tau_1, features.loc[0, 'Voltage_1:tau_charge'], places=5)
        self.assertAlmostEqual(tau_1, features.loc[0, 'Voltage_1:tau_energy'], places=5)
        self.assertAlmostEqual(tau_1, features.loc[0, 'Voltage_1:tau_lin_reg'], places=5)
        self.assertAlmostEqual(tau_1, features.loc[0, 'Voltage_1:tau_exp_fit'], places=5)

        self.assertAlmostEqual(tau_2, features.loc[0, 'Voltage_2:tau_charge'], places=5)
        self.assertAlmostEqual(tau_2, features.loc[0, 'Voltage_2:tau_energy'], places=5)
        self.assertAlmostEqual(tau_2, features.loc[0, 'Voltage_2:tau_lin_reg'], places=5)
        self.assertAlmostEqual(tau_2, features.loc[0, 'Voltage_2:tau_exp_fit'], places=5)

    def test_calculate_features_nested_dfs(self):
        # arrange
        x = np.arange(100) * 0.001
        tau_1 = 0.8
        df_1 = pd.DataFrame(data=np.exp(-x / tau_1), index=x, columns=["Voltage_1"])
        tau_2 = 0.4
        df_2 = pd.DataFrame(data=np.exp(-x / tau_2), index=x, columns=["Voltage_2"])

        # act
        function_names_to_apply = ["tau_charge", "tau_energy", "tau_lin_reg", "tau_exp_fit"]
        features = FeatureBuilder().with_signal([df_1, df_2]).calculate_features(function_names_to_apply)

        # assert
        self.assertAlmostEqual(tau_1, features.loc[0, 'Voltage_1:tau_charge'], places=5)
        self.assertAlmostEqual(tau_1, features.loc[0, 'Voltage_1:tau_energy'], places=5)
        self.assertAlmostEqual(tau_1, features.loc[0, 'Voltage_1:tau_lin_reg'], places=5)
        self.assertAlmostEqual(tau_1, features.loc[0, 'Voltage_1:tau_exp_fit'], places=5)

        self.assertAlmostEqual(tau_2, features.loc[0, 'Voltage_2:tau_charge'], places=5)
        self.assertAlmostEqual(tau_2, features.loc[0, 'Voltage_2:tau_energy'], places=5)
        self.assertAlmostEqual(tau_2, features.loc[0, 'Voltage_2:tau_lin_reg'], places=5)
        self.assertAlmostEqual(tau_2, features.loc[0, 'Voltage_2:tau_exp_fit'], places=5)
